% Mide Technology Corporation
% Code to support Dynamic Time Warping blog results
% 15 Dec 2016
% Jeff Court
% Description: an earlier blog showed that pressure sensor data could be
%   used to estimate elevation.  This builds on that blog and uses an
%   elevation history (based on google maps route) and dynamic time warping
%   to show that the path of the SSX/pressure sensor was similar to the route
%   taken from google maps.

clc; clear all; hold off; close all;    % make sure the workspace is fresh

load time_pressure                  % Load the recording data
time = time - time(1);              % Adjust the time vector to start at '0' days
SSX_Data = [[time]';[pressure]'];   % Store into a single matrix
clear time pressure                 % Remove variables no longer used

rout_info = csvread('route_Info_Full.csv')';    % [[Lat];[Long];[Ele];[Total Dist]]
rout_info(3,:) = rout_info(3,:) * 0.3048;       % Convert the elevation to meters
TestMat = SSX_Data(:,~any(isnan(SSX_Data)));    % matrix of the test data

% convert pressure to elevation - assume roughly linear
% Data taken from Mide's air pressure calculator
% 23C, 101.325 kPa, 0 m
% 23C, 80.030 kPa, 2000 m
atm = 101325;                               % atm pressure (Pa)
Pa_per_meter = (80030 - atm) / (2000);      % Pa per meter
adj = 70;   % [meters] manual adjustment to account for pressure sensor calibration, atm pressure fluctuations, etc
TestMat(3,:) = (TestMat(2,:) - atm) / Pa_per_meter + adj;   % save data as a new vector

% Check the results by plotting
figure(); 
subplot(3,1,1);                 % plot the raw pressure data
plot(SSX_Data(1,:),SSX_Data(2,:))
    xlabel('time [days]'); ylabel('pressure [Pa]'); title('Pressure Sensor Data');
subplot(3,1,2);                 % Fill in the pressure data
plot(TestMat(1,:),TestMat(2,:))
	xlabel('time [days]'); ylabel('pressure [Pa]'); title('Pressure Sensor Data');  
subplot(3,1,3);                 % Convert pressure data to elevation estimate
plot(TestMat(1,:),TestMat(3,:))
    xlabel('time [days]'); ylabel('pressure-estimated elevation [m]'); 

% Reduce the dimension of the vectors
N=500;  % factor for reduction (lower number = more computational power needed)
Red_Est = TestMat(:,1:N:length(TestMat(:,:)));  % reduced elevation estimates
Red_rout_info = rout_info(:,1:N:length(rout_info));     % reduced route info
rout_info(4,2:end) = rout_info(4,2:end) - rout_info(4,1:end-1); % change to delta miles
    % instead of total miles travelled so we can estimate miles per hour
% in the reduction, we will want to make sure we get all of the 'delta miles' parameters
for i = 1:length(Red_rout_info)         % loop over all the data points for reduced dimension
    a = 1+(i-1)*N;                      % start index of calculation
    b = min(a+N-1,length(rout_info));   % end index of calculation
    Red_rout_info(4,i) = sum(rout_info(4,a:b)); % add all of the distnaces
end
% make sure we didn't loose any distance data
oldDist = sum(rout_info(4,:)); newDist = sum(Red_rout_info(4,:));
if (oldDist ~= newDist) % verify that the total miles before and after reduction match
    disp('distance calculation error')
end

% Plot the pressure-estimated elevation and routed elevation data (all raw data)
figure(); 
subplot(2,1,1); plot(TestMat(3,:));     % test data
    xlabel('data index (estimated elevation)'); ylabel('elevation [m]'); title('Pressure Estimated Elevation');  
subplot(2,1,2); plot(rout_info(3,:));   % rout based data
    xlabel('data index (routed elevation)'); ylabel('elevation [m]'); title('Route Based Elevation');

% Plot the Original and reduced dimensional data
figure(); % This figure will have the pressure-estimated elevtion data
% first, normalize the indexes so that we plot from 0 to 1 for both data vectors
index1 = 1:length(TestMat(3,:)); index1 = index1/length(TestMat(3,:));
index2 = 1:length(Red_Est(3,:)); index2 = index2/length(Red_Est(3,:));
plot(index1,TestMat(3,:)); hold on; plot(index2,Red_Est(3,:));
ylim([0 3000]); legend('Original Data', 'Reduced Dimension Data'); 
xlabel('data vector length'); ylabel('elevation [m]'); title('Estimated Elevation Data Reduction');
figure();   % This figure will have the route based elevation data
index1 = 1:length(rout_info(3,:)); index1 = index1/length(rout_info(3,:));
index2 = 1:length(Red_rout_info(3,:)); index2 = index2/length(Red_rout_info(3,:));
plot(index1,rout_info(3,:)); hold on; plot(index2,Red_rout_info(3,:));
ylim([0 3000]); legend('Original Data', 'Reduced Dimension Data'); 
xlabel('data vector length'); ylabel('elevation [m]'); title('Routed Elevation Data Reduction');
clear index1 index2                 % Remove variables no longer used

% by visual inspection, pick off these points that we think match, when N=500
Est_Points = [60 224 329 453 502 538];
Act_Points = [10 126 196 230 286 399];
% plot the two curves and mark the points that match so that DTW can be
%   better visualized.
text_note = {' A', ' B', ' C', ' D', ' E', ' F'};   % used to label points
figure(); 
subplot(2,1,1); plot(Red_Est(3,:)); % plot the pressure data
    for i = 1:length(Est_Points)    % loop over all of the manually selected points         
        SP=Est_Points(i);           % 'x' location of the vertical line we will plot
        line([SP SP],[0 3000],'Color',[1 0 0])  % draw the line
        text(SP,2500,text_note{i})  % add the text labels
    end
    xlabel('data index (estimated elevation)'); ylabel('elevation [m]'); 
subplot(2,1,2); plot(Red_rout_info(3,:)); % plot the pressure data
    for i = 1:length(Est_Points)    % loop over all of the manually selected points 
        SP=Act_Points(i);           % 'x' location of the vertical line we will plot
        line([SP SP],[0 3000],'Color',[1 0 0])  % draw the line
        text(SP,2500,text_note{i})  % add the text labels
    end
    xlabel('data index (routed elevation)'); ylabel('elevation [m]');

% Compute the Dynamic Time Warping with the subroutine
[Dist,D,k,w]=dtw(Red_Est(3,:),Red_rout_info(3,:));

% Display the DTW path.  This page has a nice coloring solution
    % http://www.ee.columbia.edu/ln/rosa/matlab/dtw/
figure(); imagesc(D); colormap(1-gray)  % plot and color
hold on; plot(w(:,2),w(:,1),'r'); hold off  % add teh optimatl path
xlabel('data index (routed elevation)'); ylabel('data index (estimated elevation)');
% Add the points chosen through visual inspection
hold on;                            % we want to plot on the same figure
plot(Act_Points,Est_Points,'ob')    % plot the data points
hold off;                           % we will not plot on this figure again
legend('Optimal Path', 'Visual Inspection Points') 

w = flipud(w);              % adjust the optimal path matrix to be ascending order

% Generate new data matrix from all warped indexes
for i = 1:length(w)
TW_Data(i,:) = [[Red_Est(3,w(i,1))];[Red_rout_info(3,w(i,2))]];
end
% plot all of the warped data
figure(); 
plot(TW_Data(:,1)); hold on; plot(TW_Data(:,2));
xlabel('data index from optimal path'); ylabel('elevation [m]'); 
legend('Pressure-Estimated Elevation', 'Route Based Elevation'); 
title('Overlay of Time-Warped Vectors');

return
% Latitude/longitude estimates with time
% Find these times (measured in days)
Progress_Times = [1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0 11.0 12.0 13.0 14.0]'
for i = 1:length(Progress_Times)    % loop over all times of interest
    % Find the closest index in the pressure data
    [c index(i)] = min(abs(Red_Est(1,:) - Progress_Times(i)))
    % Match the index with the warping matrix 
    [lia,locb(i)] = ismember(index(i),w(:,1));
    new_Index = w(locb(i),2)
    % use the new index to extract the latitude and longitude to store in new matrix
    LatLong(i,1) = Red_rout_info(1,new_Index);
    LatLong(i,2) = Red_rout_info(2,new_Index);
end

% just want to output this data in text form
Progress_Times
LatLong



